@@ -11,11 +11,9 @@
 #   under the License.
 #
 
-from unittest import mock
 from unittest.mock import call
 import uuid
 
-from openstack.network.v2 import _proxy
 from openstack.network.v2 import (
     default_security_group_rule as _default_security_group_rule,
 )
@@ -28,18 +26,7 @@ from openstackclient.tests.unit.network.v2 import fakes as network_fakes
 from openstackclient.tests.unit import utils as tests_utils
 
 
-class TestDefaultSecurityGroupRule(network_fakes.TestNetworkV2):
-    def setUp(self):
-        super().setUp()
-
-        self.app.client_manager.sdk_connection = mock.Mock()
-        self.app.client_manager.sdk_connection.network = mock.Mock(
-            spec=_proxy.Proxy,
-        )
-        self.sdk_client = self.app.client_manager.sdk_connection.network
-
-
-class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
+class TestCreateDefaultSecurityGroupRule(network_fakes.TestNetworkV2):
     expected_columns = (
         'description',
         'direction',
@@ -82,7 +69,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
             **default_security_group_rule_attrs,
         )
 
-        self.sdk_client.create_default_security_group_rule.return_value = (
+        self.network_client.create_default_security_group_rule.return_value = (
             self._default_sg_rule
         )
         self.expected_data = (
@@ -208,7 +195,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -252,7 +239,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -302,7 +289,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -347,7 +334,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -381,7 +368,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -415,7 +402,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -449,7 +436,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -599,7 +586,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -637,7 +624,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -675,7 +662,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -713,7 +700,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -754,7 +741,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -794,7 +781,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'direction': self._default_sg_rule.direction,
                 'ethertype': self._default_sg_rule.ether_type,
@@ -825,7 +812,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.create_default_security_group_rule.assert_called_once_with(
+        self.network_client.create_default_security_group_rule.assert_called_once_with(
             **{
                 'description': self._default_sg_rule.description,
                 'direction': self._default_sg_rule.direction,
@@ -840,7 +827,7 @@ class TestCreateDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
         self.assertEqual(self.expected_data, data)
 
 
-class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
+class TestDeleteDefaultSecurityGroupRule(network_fakes.TestNetworkV2):
     # The default security group rules to be deleted.
     default_security_group_rule_attrs = {
         'direction': 'ingress',
@@ -866,7 +853,9 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
     def setUp(self):
         super().setUp()
 
-        self.sdk_client.delete_default_security_group_rule.return_value = None
+        self.network_client.delete_default_security_group_rule.return_value = (
+            None
+        )
 
         # Get the command object to test
         self.cmd = default_security_group_rule.DeleteDefaultSecurityGroupRule(
@@ -880,7 +869,7 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
         verifylist = [
             ('rule', [self._default_sg_rules[0].id]),
         ]
-        self.sdk_client.find_default_security_group_rule.return_value = (
+        self.network_client.find_default_security_group_rule.return_value = (
             self._default_sg_rules[0]
         )
 
@@ -888,7 +877,7 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         result = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.delete_default_security_group_rule.assert_called_once_with(
+        self.network_client.delete_default_security_group_rule.assert_called_once_with(
             self._default_sg_rules[0]
         )
         self.assertIsNone(result)
@@ -902,7 +891,7 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
         verifylist = [
             ('rule', arglist),
         ]
-        self.sdk_client.find_default_security_group_rule.side_effect = (
+        self.network_client.find_default_security_group_rule.side_effect = (
             self._default_sg_rules
         )
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -912,7 +901,7 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
         calls = []
         for s in self._default_sg_rules:
             calls.append(call(s))
-        self.sdk_client.delete_default_security_group_rule.assert_has_calls(
+        self.network_client.delete_default_security_group_rule.assert_has_calls(
             calls
         )
         self.assertIsNone(result)
@@ -931,8 +920,8 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
             self._default_sg_rules[0],
             exceptions.CommandError,
         ]
-        self.sdk_client.find_default_security_group_rule = mock.Mock(
-            side_effect=find_mock_result
+        self.network_client.find_default_security_group_rule.side_effect = (
+            find_mock_result
         )
 
         try:
@@ -941,18 +930,18 @@ class TestDeleteDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
         except exceptions.CommandError as e:
             self.assertEqual('1 of 2 default rules failed to delete.', str(e))
 
-        self.sdk_client.find_default_security_group_rule.assert_any_call(
+        self.network_client.find_default_security_group_rule.assert_any_call(
             self._default_sg_rules[0].id, ignore_missing=False
         )
-        self.sdk_client.find_default_security_group_rule.assert_any_call(
+        self.network_client.find_default_security_group_rule.assert_any_call(
             'unexist_rule', ignore_missing=False
         )
-        self.sdk_client.delete_default_security_group_rule.assert_called_once_with(
+        self.network_client.delete_default_security_group_rule.assert_called_once_with(
             self._default_sg_rules[0]
         )
 
 
-class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
+class TestListDefaultSecurityGroupRule(network_fakes.TestNetworkV2):
     # The security group rule to be listed.
     _default_sg_rule_tcp = sdk_fakes.generate_fake_resource(
         _default_security_group_rule.DefaultSecurityGroupRule,
@@ -1001,7 +990,7 @@ class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
     def setUp(self):
         super().setUp()
 
-        self.sdk_client.default_security_group_rules.return_value = (
+        self.network_client.default_security_group_rules.return_value = (
             self._default_sg_rules
         )
 
@@ -1016,7 +1005,7 @@ class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.default_security_group_rules.assert_called_once_with(
+        self.network_client.default_security_group_rules.assert_called_once_with(
             **{}
         )
         self.assertEqual(self.expected_columns, columns)
@@ -1035,7 +1024,7 @@ class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.default_security_group_rules.assert_called_once_with(
+        self.network_client.default_security_group_rules.assert_called_once_with(
             **{
                 'protocol': 'tcp',
             }
@@ -1055,7 +1044,7 @@ class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.default_security_group_rules.assert_called_once_with(
+        self.network_client.default_security_group_rules.assert_called_once_with(
             **{
                 'direction': 'ingress',
             }
@@ -1075,7 +1064,7 @@ class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.default_security_group_rules.assert_called_once_with(
+        self.network_client.default_security_group_rules.assert_called_once_with(
             **{
                 'direction': 'egress',
             }
@@ -1084,7 +1073,7 @@ class TestListDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
         self.assertEqual(self.expected_data, list(data))
 
 
-class TestShowDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
+class TestShowDefaultSecurityGroupRule(network_fakes.TestNetworkV2):
     # The default security group rule to be shown.
     _default_sg_rule = sdk_fakes.generate_fake_resource(
         _default_security_group_rule.DefaultSecurityGroupRule
@@ -1123,7 +1112,7 @@ class TestShowDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
     def setUp(self):
         super().setUp()
 
-        self.sdk_client.find_default_security_group_rule.return_value = (
+        self.network_client.find_default_security_group_rule.return_value = (
             self._default_sg_rule
         )
 
@@ -1148,7 +1137,7 @@ class TestShowDefaultSecurityGroupRule(TestDefaultSecurityGroupRule):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.sdk_client.find_default_security_group_rule.assert_called_once_with(
+        self.network_client.find_default_security_group_rule.assert_called_once_with(
             self._default_sg_rule.id, ignore_missing=False
         )
         self.assertEqual(self.columns, columns)
