@@ -18,15 +18,18 @@ import argparse
 import copy
 import functools
 import logging
+import typing as ty
 
 from cliff import columns as cliff_columns
+from openstack.block_storage.v2 import volume as _volume
 from openstack import exceptions as sdk_exceptions
 from osc_lib.cli import format_columns
 from osc_lib.cli import parseractions
-from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
 
+from openstackclient.api import volume_v2
+from openstackclient import command
 from openstackclient.common import pagination
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
@@ -58,7 +61,7 @@ class KeyValueHintAction(argparse.Action):
             )
 
 
-class AttachmentsColumn(cliff_columns.FormattableColumn):
+class AttachmentsColumn(cliff_columns.FormattableColumn[list[ty.Any]]):
     """Formattable column for attachments column.
 
     Unlike the parent FormattableColumn class, the initializer of the
@@ -89,6 +92,47 @@ class AttachmentsColumn(cliff_columns.FormattableColumn):
         return msg
 
 
+def _format_volume(volume: _volume.Volume) -> dict[str, ty.Any]:
+    # Some columns returned by openstacksdk should not be shown because they're
+    # either irrelevant or duplicates
+    ignored_columns = {
+        # computed columns
+        'location',
+        # create-only columns
+        'OS-SCH-HNT:scheduler_hints',
+        'imageRef',
+        # unnecessary columns
+        'links',
+    }
+    optional_columns = {
+        # only present if part of a consistency group
+        'consistencygroup_id',
+        # only present if there are image properties associated
+        'volume_image_metadata',
+    }
+
+    info = volume.to_dict(original_names=True)
+    data = {}
+    for key, value in info.items():
+        if key in ignored_columns:
+            continue
+
+        if key in optional_columns:
+            if info[key] is None:
+                continue
+
+        data[key] = value
+
+    data.update(
+        {
+            'properties': format_columns.DictColumn(data.pop('metadata')),
+            'type': data.pop('volume_type'),
+        }
+    )
+
+    return data
+
+
 class CreateVolume(command.ShowOne):
     _description = _("Create new volume")
 
@@ -226,22 +270,22 @@ class CreateVolume(command.ShowOne):
         # volume from snapshot or source volume
         size = parsed_args.size
 
-        volume_client = self.app.client_manager.volume
+        volume_client = self.app.client_manager.sdk_connection.volume
         image_client = self.app.client_manager.image
 
         source_volume = None
         if parsed_args.source:
-            source_volume_obj = utils.find_resource(
-                volume_client.volumes, parsed_args.source
+            source_volume_obj = volume_client.find_volume(
+                parsed_args.source, ignore_missing=False
             )
             source_volume = source_volume_obj.id
             size = max(size or 0, source_volume_obj.size)
 
         consistency_group = None
         if parsed_args.consistency_group:
-            consistency_group = utils.find_resource(
-                volume_client.consistencygroups, parsed_args.consistency_group
-            ).id
+            consistency_group = volume_v2.find_consistency_group(
+                volume_client, parsed_args.consistency_group
+            )['id']
 
         image = None
         if parsed_args.image:
@@ -251,8 +295,8 @@ class CreateVolume(command.ShowOne):
 
         snapshot = None
         if parsed_args.snapshot:
-            snapshot_obj = utils.find_resource(
-                volume_client.volume_snapshots, parsed_args.snapshot
+            snapshot_obj = volume_client.find_snapshot(
+                parsed_args.snapshot, ignore_missing=False
             )
             snapshot = snapshot_obj.id
             # Cinder requires a value for size when creating a volume
@@ -263,7 +307,7 @@ class CreateVolume(command.ShowOne):
             # snapshot size.
             size = max(size or 0, snapshot_obj.size)
 
-        volume = volume_client.volumes.create(
+        volume = volume_client.create_volume(
             size=size,
             snapshot_id=snapshot,
             name=parsed_args.name,
@@ -271,23 +315,23 @@ class CreateVolume(command.ShowOne):
             volume_type=parsed_args.type,
             availability_zone=parsed_args.availability_zone,
             metadata=parsed_args.properties,
-            imageRef=image,
-            source_volid=source_volume,
-            consistencygroup_id=consistency_group,
+            image_id=image,
+            source_volume_id=source_volume,
+            consistency_group_id=consistency_group,
             scheduler_hints=parsed_args.hint,
         )
 
         if parsed_args.bootable is not None:
             try:
                 if utils.wait_for_status(
-                    volume_client.volumes.get,
+                    volume_client.get_volume,
                     volume.id,
                     success_status=['available'],
                     error_status=['error'],
                     sleep_time=1,
                 ):
-                    volume_client.volumes.set_bootable(
-                        volume.id, parsed_args.bootable
+                    volume_client.set_volume_bootable_status(
+                        volume, parsed_args.bootable
                     )
                 else:
                     msg = _(
@@ -300,14 +344,14 @@ class CreateVolume(command.ShowOne):
         if parsed_args.read_only is not None:
             try:
                 if utils.wait_for_status(
-                    volume_client.volumes.get,
+                    volume_client.get_volume,
                     volume.id,
                     success_status=['available'],
                     error_status=['error'],
                     sleep_time=1,
                 ):
-                    volume_client.volumes.update_readonly_flag(
-                        volume.id, parsed_args.read_only
+                    volume_client.set_volume_readonly(
+                        volume, parsed_args.read_only
                     )
                 else:
                     msg = _(
@@ -321,17 +365,8 @@ class CreateVolume(command.ShowOne):
                     e,
                 )
 
-        # Remove key links from being displayed
-        volume._info.update(
-            {
-                'properties': format_columns.DictColumn(
-                    volume._info.pop('metadata')
-                ),
-                'type': volume._info.pop('volume_type'),
-            }
-        )
-        volume._info.pop("links", None)
-        return zip(*sorted(volume._info.items()))
+        data = _format_volume(volume)
+        return zip(*sorted(data.items()))
 
 
 class DeleteVolume(command.Command):
@@ -355,12 +390,19 @@ class DeleteVolume(command.Command):
             ),
         )
         group.add_argument(
-            "--purge",
+            "--cascade",
             action="store_true",
             help=_(
                 "Remove any snapshots along with volume(s) (defaults to False)"
             ),
         )
+        group.add_argument(
+            # now called "cascade", accept old arg for compatibility
+            "--purge",
+            action="store_true",
+            help=argparse.SUPPRESS,
+            dest='cascade',
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -375,7 +417,7 @@ class DeleteVolume(command.Command):
                 volume_client.delete_volume(
                     volume_obj.id,
                     force=parsed_args.force,
-                    cascade=parsed_args.purge,
+                    cascade=parsed_args.cascade,
                 )
             except Exception as e:
                 result += 1
@@ -578,13 +620,15 @@ class MigrateVolume(command.Command):
         return parser
 
     def take_action(self, parsed_args):
-        volume_client = self.app.client_manager.volume
-        volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
-        volume_client.volumes.migrate_volume(
+        volume_client = self.app.client_manager.sdk_connection.volume
+        volume = volume_client.find_volume(
+            parsed_args.volume, ignore_missing=False
+        )
+        volume_client.migrate_volume(
             volume.id,
-            parsed_args.host,
-            parsed_args.force_host_copy,
-            parsed_args.lock_volume,
+            host=parsed_args.host,
+            force_host_copy=parsed_args.force_host_copy,
+            lock_volume=parsed_args.lock_volume,
         )
 
 
@@ -873,12 +917,12 @@ class SetVolume(command.Command):
         elif policy:
             # If the "--migration-policy" is specified without "--type"
             LOG.warning(
-                _("'%s' option will not work without '--type' option")
-                % (
+                _("'%s' option will not work without '--type' option"),
+                (
                     '--migration-policy'
                     if parsed_args.migration_policy
                     else '--retype-policy'
-                )
+                ),
             )
 
         kwargs = {}
@@ -918,24 +962,13 @@ class ShowVolume(command.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
-        volume_client = self.app.client_manager.volume
-        volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
-
-        # Special mapping for columns to make the output easier to read:
-        # 'metadata' --> 'properties'
-        # 'volume_type' --> 'type'
-        volume._info.update(
-            {
-                'properties': format_columns.DictColumn(
-                    volume._info.pop('metadata')
-                ),
-                'type': volume._info.pop('volume_type'),
-            },
+        volume_client = self.app.client_manager.sdk_connection.volume
+        volume = volume_client.find_volume(
+            parsed_args.volume, ignore_missing=False
         )
 
-        # Remove key links from being displayed
-        volume._info.pop("links", None)
-        return zip(*sorted(volume._info.items()))
+        data = _format_volume(volume)
+        return zip(*sorted(data.items()))
 
 
 class UnsetVolume(command.Command):
